function [N, eta, gamma] = RobustSyncOptimalVars(A, B, C, L, delta, ssigma, tau)
% [N, eta, gamma] = RobustSyncOptimalvars(A, B, C, L)
% gives choices for N, eta and gamma maximising eta using section 6 of the article.
% Default values for are used for delta, and sigma and tau.
% 
% [N, eta, gamma] = RobustSyncOptimalVars(A, B, C, L, delta, sigma, tau)
% gives optimal choices for N, eta and gamma maximising eta.

  %select default values for delta, sigma, tau
  if nargin < 5
    delta = 10^-6;
    ssigma = 10^-6;
    tau = 10^-6;
  end

  % compute the eigenvalues the graph laplacian.
  E = sort(real(eig(L)));
  lambda2 = E(2);
  lambdaP = E(end);
  
  % calculate P and Q as given by (48) and (49)
  P = care(A, B, ssigma*eye(size(A)));
  Q = care(A', C', tau*eye(size(A')));
  
  eta = lambda2 / sqrt(lambdaP^2 + 2 * delta)  * 1/sqrt(max(abs(eig(P*Q)))) - delta;
  gamma = lambda2^2/(lambdaP^2 + 2*delta);
  N = (lambdaP^2 + delta)/lambda2;
  
  %testing if the found numbers are sane
  if ~((0 < eta) && (eta < (1/sqrt(max(abs(eig(P*Q)))))))
    warning('eta out of bounds. Choose a smaller value delta');
  end
  if N <= lambdaP^2/lambda2
    warning('N out of bounds. Choose a smaller value delta');
  end
  if ~(( lambdaP^2/N^2 <= gamma) && (gamma  < lambda2/N))
    warning('gamma out of bounds. Choose a smaller value delta');
  end
end